unit global_hotkeys;

interface

uses
  Windows;

type
  TGlobalHotkeyProc = procedure(KeyId : integer);

function UnregisterGlobalHotKey(KeyId : integer) : boolean;
function RegisterGlobalHotKey(KeyId : integer; Modifiers, vkCode : UINT;
  KeyProc : TGlobalHotkeyProc) : boolean;

implementation

const
  HotKeysCount = 100;
  HotKeyPollInterval = 100;

type
  TKeyCombo = array[0..4] of WORD;
  THotKeyItem = record
    Id : integer;
    Keys : TKeyCombo;
    WasPressedBefore : boolean;
    KeyProc : TGlobalHotkeyProc;
    fuModifiers, uVirtKey : WORD;
  end;

  TCriticalSection = class(TObject)
  protected
    FSection : TRTLCriticalSection;
  public
    procedure Enter;
    procedure Leave;
    constructor Create;
    destructor Destroy; override;
  end;

var
  ThreadId : Cardinal;
  HotKeyThreadH : THandle;
  CSKeys : TCriticalSection;
  ThreadTerminated : boolean;
  HotKeyListIndex : integer = -1;
  HotKeyList : array[0..HotKeysCount - 1] of THotKeyItem;

type
  TKeyState = (ks_Undefined = 0, ks_Pressed = 1, ks_NotPressed = 2);

var
  KeyState : array[0..255] of TKeyState;

procedure TCriticalSection.Enter;
begin
  EnterCriticalSection(FSection);
end;

procedure TCriticalSection.Leave;
begin
  LeaveCriticalSection(FSection);
end;

constructor TCriticalSection.Create;
begin
  InitializeCriticalSection(FSection);
end;

destructor TCriticalSection.Destroy;
begin
  DeleteCriticalSection(FSection);
  inherited Destroy;
end;

function IsKeyPressed(Key : integer) : boolean;
var
  KS : DWORD;
begin
  if Key > 255 then
  begin
    Result := ((Word(GetAsyncKeyState(Key)) shr 15) and 1) = 1;
    Exit;
  end;
  if Keystate[Key] = ks_Undefined then
  begin
    KS := GetAsyncKeyState(Key);
    if ((KS and 1) = 1) then
    begin
      Keystate[Key] := ks_Pressed;
    end
    else
    begin
      if ((KS shr 15) and 1) = 1 then
      begin
        Keystate[Key] := ks_Pressed;
      end
      else
      begin
        Keystate[Key] := ks_NotPressed;
      end;
    end;
  end;
  Result := Keystate[Key] = ks_Pressed;
end;

procedure ClearKeyState;
var
  I : integer;
begin
  zeromemory(@Keystate[0], 256 * SizeOf(TKeyState));
  for i := 0 to 255 do
  begin
    GetAsyncKeyState(I);
  end;
end;

function CheckKeyCombo(Keycombo : TKeyCombo) : boolean;
var
  i : integer;
begin
  Result := False;
  if Keycombo[0] <> 0 then
  begin
    Result := True;
    for I := 0 to Length(Keycombo) - 1 do
    begin
      if (Keycombo[I] = 0) then
      begin
        Exit;
      end;
      if not IsKeyPressed(Keycombo[I]) then
      begin
        Result := False;
        Exit;
      end;
    end;
  end;
end;

procedure ConvertOldHotKeyToKeyCombo(fsModifiers, VK : uint; var K : TKeyCombo);
var
  I : integer;
begin
  K[0] := VK;
  I := 1;
  if (fsModifiers and MOD_CONTROL) > 0 then
  begin
    K[I] := VK_CONTROL;
    i := I + 1;
  end;
  if (fsModifiers and MOD_ALT) > 0 then
  begin
    K[I] := VK_MENU;
    i := I + 1;
  end;
  if (fsModifiers and MOD_SHIFT) > 0 then
  begin
    k[I] := VK_SHIFT;
    i := I + 1;
  end;
  K[I] := 0;
end;

procedure HotKeyTheadProc;
var
  I : integer;
begin
  while not ThreadTerminated do
  begin
    try
      CSKeys.Enter;
      try
        for I := 0 to HotKeyListIndex do
        begin
          if CheckKeyCombo(HotKeyList[I].Keys) then
          begin
            if not HotKeyList[I].WasPressedBefore then
            begin
              if @HotKeyList[I].KeyProc <> nil then
              begin
                HotKeyList[I].KeyProc(HotKeyList[I].Id);
              end;
              HotKeyList[I].WasPressedBefore := True;
            end;
          end
          else
          begin
            HotKeyList[I].WasPressedBefore := False;
          end;
        end;
      finally
        CSKeys.Leave;
      end;
    except
    end;
    ClearKeyState;
    Sleep(HotKeyPollInterval);
  end;
end;

procedure EndWorkOfGlobalHotKeys;
begin
  if HotKeyThreadH > 0 then
  begin
    ThreadTerminated := True;
    SuspendThread(HotKeyThreadH);
    CloseHandle(HotKeyThreadH);
  end;
  if CSKeys <> nil then
  begin
    CSKeys.Free;
  end;
end;

procedure StartWorkOfGlobalHotKeys;
begin
  ThreadTerminated := False;
  CSKeys := TCriticalSection.Create;
  HotKeyThreadH := CreateThread(nil, 0, @HotKeyTheadProc, nil, 0, ThreadId);
end;

function RegisterGlobalHotKey(KeyId : integer; Modifiers, vkCode : UINT;
  KeyProc : TGlobalHotkeyProc) : boolean;
begin
  CSKeys.Enter;
  Result := False;
  try
    if HotKeyListIndex = -1 then
    begin
      if HotKeyThreadH = 0 then
      begin
        StartWorkOfGlobalHotKeys;
      end;
    end;
    if HotKeyListIndex < High(HotKeyList) then
    begin
      HotKeyListIndex := HotKeyListIndex + 1;
      HotKeyList[HotKeyListIndex].Id := KeyId;
      HotKeyList[HotKeyListIndex].uVirtKey := vkCode;
      HotKeyList[HotKeyListIndex].WasPressedBefore := False;
      HotKeyList[HotKeyListIndex].fuModifiers := Modifiers;
      if @KeyProc <> nil then
      begin
        HotKeyList[HotKeyListIndex].KeyProc := KeyPRoc;
      end;
      ConvertOldHotKeyToKeyCombo(Modifiers, vkCode, HotKeyList[HotKeyListIndex].Keys);
      Result := True;
    end;
  finally
    CSKeys.Leave;
  end;
end;

function UnregisterGlobalHotKey(KeyId : integer) : boolean;
var
  I, J : integer;
  Found : boolean;
begin
  Found := False;
  CSKeys.Enter;
  try
    for i := 0 to Length(HotKeyList) - 1 do
    begin
      if HotKeyList[I].Id = KeyId then
      begin
        Found := True;
        for J := I to Length(HotKeyList) - 2 do
        begin
          HotKeyList[J] := HotKeyList[J + 1];
        end;
        Break;
      end;
    end;
    Result := Found;
    if Result then
    begin
      HotKeyListIndex := HotKeyListIndex - 1;
    end;
  finally
    CSKeys.Leave;
    if HotKeyListIndex = -1 then
    begin
      EndWorkOfGlobalHotKeys;
    end;
  end;
end;

initialization
  begin
    ClearKeyState;
    StartWorkOfGlobalHotKeys;
  end;

finalization
  begin
    EndWorkOfGlobalHotKeys;
  end;
end.