unit xmpfunc;

interface

uses
  Windows;

type
  QWORD = LongWord;
  TInterfaceProc = function(Face : DWORD) : Pointer; stdcall;

const
  XMPFUNC_MISC_FACE = 0; // miscellaneous functions (XMPFUNC_MISC)
  XMPFUNC_REGISTRY_FACE = 1; // registry functions (XMPFUNC_REGISTRY)
  XMPFUNC_FILE_FACE = 2; // file functions (XMPFUNC_FILE)
  XMPFUNC_TEXT_FACE = 3; // text functions (XMPFUNC_TEXT)
  XMPFUNC_STATUS_FACE = 4; // playback status functions (XMPFUNC_STATUS)

  XMPCONFIG_NET_BUFFER = 0;
  XMPCONFIG_NET_RESTRICT = 1;
  XMPCONFIG_NET_RECONNECT = 2;
  XMPCONFIG_NET_PROXY = 3;
  XMPCONFIG_NET_PROXYCONF = 4;
  XMPCONFIG_NET_TIMEOUT = 5;
  XMPCONFIG_NET_PREBUF = 6;

  XMPINFO_TEXT_GENERAL = 0; // General info text
  XMPINFO_TEXT_MESSAGE = 1; // Message info text
  XMPINFO_TEXT_SAMPLES = 2; // Samples info text

  XMPINFO_REFRESH_MAIN = 1; // main window info area
  XMPINFO_REFRESH_GENERAL = 2; // General info window
  XMPINFO_REFRESH_MESSAGE = 4; // Message info window
  XMPINFO_REFRESH_SAMPLES = 8; // Samples info window

type
  XMPFILE = Pointer;

const
  XMPFILE_TYPE_MEMORY = 0; // file in memory
  XMPFILE_TYPE_FILE = 1; // local file
  XMPFILE_TYPE_NETFILE = 2; // file on the 'net
  XMPFILE_TYPE_NETSTREAM = 3; // 'net stream (unknown length)

type
  XMPSHORTCUT = packed record
    ID : DWORD; // must be unique and >=0x10000
    Char : PAnsiChar; // description
    Proc : Pointer; // handler
  end;

type
  XMPFORMAT = packed record
    Rate : DWORD; // sample rate
    CHand : DWORD; // channels
    Res : DWORD; // bytes per sample (1=8-bit,2=16-bit,3=24-bit,4=float,0=undefined)
  end;

  XMPCUE = packed record
    Time : Single; // cue position
    Title : PAnsiChar;
    Performer : PAnsiChar;
  end;

  PXMPSHORTCUT = ^XMPSHORTCUT;

  TGetVersion = function : DWORD; stdcall;
  TGetWindow = function : HWND; stdcall;
  TAlloc = procedure(Len : DWORD); stdcall;
  TReAlloc = procedure(Mem : Pointer; Len : DWORD); stdcall;
  TFree = procedure(Mem : Pointer); stdcall;
  TCheckCancel = function : DWORD; stdcall;
  TGetConfig = function(Option : DWORD) : DWORD; stdcall;
  TGetSkinConfig = function(const Name : PAnsiChar) : PAnsiChar; stdcall;
  TShowBubble = procedure(const Text : PAnsiChar; Time : DWORD); stdcall;
  TRefreshInfo = procedure(Mode : DWORD); stdcall;
  TGetInfoText = function(Mode : DWORD) : PAnsiChar; stdcall;
  TFormatInfoText = function(Buf : PAnsiChar; const Name : PAnsiChar; const Value : PAnsiChar) : PAnsiChar; stdcall;
  TGetTag = function(Tag : integer) : PAnsiChar; stdcall;
  TRegisterShortcut = function(const Cut : PXMPSHORTCUT) : LongBool stdcall;
  TPerformShortcut = function(ID : DWORD) : LongBool; stdcall;
  TGetCue = function(Cue : DWORD) : XMPCUE; stdcall;

  XMPFUNC_MISC = packed record // miscellaneous functions
    GetVersion : TGetVersion; // get XMPlay version (eg. 0x03040001 = 3.4.0.1)
    GetWindow : TGetWindow; // get XMPlay window handle
    Alloc : TAlloc; // allocate memory
    ReAlloc : TReAlloc; // re-allocate memory
    Free : TFree; // free allocated memory/text
    CheckCance : TCheckCancel;
    GetConfig : TGetConfig; // get a config (XMPCONFIG_xxx) value
    GetSkinConfig : TGetSkinConfig; // get a skinconfig value
    ShowBubble : TShowBubble; // show a help bubble (time in ms, 0=default)
    RefreshInfo : TRefreshInfo; // refresh info displays (XMPINFO_REFRESH_xxx flags)
    GetInfoText : TGetInfoText; // get info window text (XMPINFO_TEXT_xxx)
    FormatInfoText : TFormatInfoText; // format text for info window (tabs & new-lines)
    GetTag : TGetTag; // get a current track's tag
                     // tags: 0=title,1=artist,2=album,3=year,4=track,5=genre,6=comment,
                     // 7=filetype, -1=formatted title,-2=filename,-3=track/cue title
    RegisterShortcut : TRegisterShortcut; // add a shortcut
    PerformShortcut : TPerformShortcut; // perform a shortcut action
  // version 3.4.0.14
    GetCue : TGetCue; // get a cue entry (0=image, 1=1st track)
  end;

  TGet = function(const Section : PAnsiChar; const Key : PAnsiChar; Data : Pointer; Size : DWORD) : DWORD; stdcall;
  TGetString = function(const Section : PAnsiChar; const Key : PAnsiChar; Data : PAnsiChar; Dize : DWORD) : DWORD; stdcall;
  TGetInt = function(const Section : PAnsiChar; const Key : PAnsiChar; Data : PInt) : LongBool; stdcall;
  TSet = function(const Section : PAnsiChar; const Key : PAnsiChar; const Data : Pointer; Size : DWord) : LongBool; stdcall;
  TSetString = function(const Section : PAnsiChar; const Key : PAnsiChar; const Data : PAnsiChar) : LongBool; stdcall;
  TSetInt = function(const Section : PAnsiChar; const Key : PAnsiChar; const Data : PInt) : LongBool; stdcall;

  XMPFUNC_REGISTRY = packed record // "registry" functions
    Get : TGet; // if data=NULL, required size is returned
    GetString : TGetString; // if data=NULL, required size is returned
    GetInt : TGetInt;
    Sets : TSet; // data=NULL = delete key
    SEtString : TSetString;
    SetInt : TSetInt;
  end;

  TOpen = function(const Filename : PAnsiChar) : XMPFILE; stdcall;
  TOpenMemory = function(const Buf : Pointer; Len : DWORD) : XMPFILE; stdcall;
  TClose = procedure(AFile : XMPFILE); stdcall;
  TGetType = function(AFile : XMPFILE) : DWORD; stdcall;
  TGetSize = function(AFile : XMPFILE) : DWORD; stdcall;
  TGetFilename = function(AFile : XMPFILE) : PAnsiChar stdcall;
  TGetMemory = procedure(AFile : XMPFILE); stdcall;
  TRead = function(AFile : XMPFILE; Buf : Pointer; Len : DWORD) : DWORD; stdcall;
  TSeek = function(AFile : XMPFILE; Pos : DWORD) : LongBool; stdcall;
  TTell = function(AFile : XMPFILE) : DWORD; stdcall;

  TNetSetRate = procedure(AFile : XMPFILE; Rate : DWORD); stdcall;
  TNetIsActive = function(AFile : XMPFILE) : LongBool; stdcall;
  TNetPreBuf = function(Afile : XMPFILE) : LongBool; stdcall;
  TNetAvailable = function(AFile : XMPFILE) : DWORD; stdcall;

  TArchiveList = function(AFile : XMPFILE) : PAnsiChar stdcall;
  TArchiveExtract = function(AFile : XMPFILE; const Entry : PAnsiChar; Len : DWORD) : XMPFILE; stdcall;

  XMPFUNC_FILE = packed record // file functions
    Open : TOpen; // open a file
    OpenMemory : TOpenMemory; // open a file from memory
    Close : TClose; // close an opened file
    GetTyoe : TGetTYpe; // return XMPFILE_TYPE_xxx
    GetSize : TGetSize; // file size
    GetFilename : TGetFilename; // filename
    GetMemory : TGetMemory; // memory location (XMPFILE_TYPE_MEMORY)
    Read : TRead; // read from file
    Seek : TSeek; // seek in file
    Tell : TTell; // get current file pos
 // net-only stuff
    NetSetRate : TNetSetRate; // set bitrate in bytes/sec (decides buffer size)
    NetIsActive : TNetIsActive; // connection is still up?
    NetPreBuf : TNetPreBuf; // pre-buffer data
    NetAvailable : TNetAvailable; // get amount of data ready to go

    ArchiveList : TArchiveList; // get archive contents (series of NULL-terminated entries)
    ArchiveExtract : TArchiveExtract; // decompress file from archive
  end;

  TAnsi = function(const Text : PAnsiChar; Len : integer) : PAnsiChar stdcall;
  TUnicode = function(const Text : PWideChar; Len : integer) : PAnsiChar stdcall;
  TUtf8 = function(const Text : PAnsiChar; Len : integer) : PAnsiChar stdcall;

  XMPFUNC_TEXT = packed record // text functions - return new string in native form (UTF-8/ANSI)
    Ansi : TAnsi; // ANSI string (len=-1=null terminated)
    Unicode : TUnicode; // Unicode string
    Utf8 : TUtf8; // UTF-8 string
  end;

  TIsPlaying = function : LongBool; stdcall;
  TGetTime = function : Double; stdcall;
  TGetWritten = function : QWORD; stdcall;
  TGetLatency = function : DWORD; stdcall;
  TGetFormat = function(Input : LongBool) : XMPFORMAT; stdcall;

  XMPFUNC_STATUS = packed record // playback status functions
    IsPlaying : TIsPlaying; // playing?
    GetTime : TGetTime; // track position in seconds
    GetWritten : TGetWritten; // samples written to output
    GetLatency : TGetLatency; // samples in output buffer
    GetFormat : TGetFormat; // get input/output sample format
  end;

implementation

end.